xf_keys = {
    "data_key": "data.block.0.line",
    "word_key": "word.0.content",
    "left_key": "location.top_left.x",
    "top_key": "location.top_left.y",
    "right_key": "location.right_bottom.x",
    "bottom_key": "location.right_bottom.y"
}
